package client;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.EOFException;
import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import javax.swing.JFrame;
import javax.swing.JOptionPane;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingUtilities;

/**
 * Třída demonstrující klienta pro komunikaci mezi programy pomocí
 * TCP.
 */
public class TCPClient extends JFrame {

    /**
     * Textová oblast zobrazující zprávy a další informace uživateli.
     */
    private JTextArea displayArea;
    /**
     * Textová oblast zobrazující obsah souboru;
     */
    private JTextArea fileArea;
    /**
     * Výstupní proud směrem k serveru.
     */
    private DataOutputStream output;
    /**
     * Vstupní proud směrem od serveru.
     */
    private DataInputStream input;
    /**
     * Adresa serveru.
     */
    private String chatServer;
    private int port;
    /**
     * Soket, pomocí kterého komunikuje klient se serverem.
     */
    private Socket client;

    /**
     * Vytvoří novou instanci třídy TCPClient a inicializuje prvky
     * uživatelského rozhraní.
     *
     * @param host
     *            adresa serveru, kam se tento klient připojí.
     */
    public TCPClient(String host, int port) {
        super("TCP Klient");
        setLayout(new BorderLayout());
        // nastavení adresy serveru
        chatServer = host;
        this.port = port;

        // vytvoření textové oblasti pro zobrazování výstupu
        displayArea = new JTextArea();
        fileArea = new JTextArea();
        add(new JScrollPane(fileArea), BorderLayout.CENTER);
        add(new JScrollPane(displayArea), BorderLayout.SOUTH);
        setSize(400, 250);
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    /**
     * Hlavní funkční metoda klienta, kde dochází k připojení k
     * serveru a obsluze tohoto spojení.
     */
    public void runClient(String fileName) {
        try {
            // vytvoření soketu pro spojení
            try {
                connectToServer();
            } catch (IOException e) {
                JOptionPane.showMessageDialog(this, 
                        "Spojení se serverem se nepodařilo navázat.\nUjistěte se, že je puštěn.",
                        "Nelze navázat spojení",
                        JOptionPane.ERROR_MESSAGE);
            }
            // získání vstupního a výstupního proudu
            getStreams();
            // zpracování spojení
            processConnection(fileName);
        } catch (EOFException e) {
            displayMessage("\nServer ukočil spojení.");
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            // uzavření spojení
            closeConnection();
        }
    }

    /**
     * Připojí se k serveru na adrese předané do konstruktoru.
     *
     * @throws IOException
     *             pokud dojde během připojování k chybě.
     */
    private void connectToServer() throws IOException {
        displayMessage("Zkouším spojení.\n");
        // vytvoření soketu a navázání spojení

        client =
                new Socket(InetAddress.getByName(chatServer), port);
        // zobrazení informace o spojení
        displayMessage("Připojeno k: "
                + client.getInetAddress().getHostName());

    }

    /**
     * Získá ze soketu proudy pro odesílání a příjem dat.
     *
     * @throws IOException
     *             pokud dojde při získávání proudů k chybě.
     */
    private void getStreams() throws IOException {
        output = new DataOutputStream(client.getOutputStream());
        input = new DataInputStream(client.getInputStream());

        displayMessage("\nZískány vstupní a výstupní proud.\n");
    }

    /**
     * Zpracování spojení se serverem.
     *
     * @param String fileName Jmeno pozadovaneho souboru
     * @throws IOException
     *             pokud při komunikaci se serverem dojde k chybě.
     */
    private void processConnection(String fileName) throws IOException {
        String message;
        // zpracování zpráv od serveru
        displayMessage("\nPožaduji soubor " + fileName);
        output.writeUTF(fileName);
        output.flush();
        byte stav = input.readByte();
        if (stav == 0) {
            displayMessage("\nSoubor nelze číst");
            JOptionPane.showMessageDialog(this, "Soubor nelze číst",
                    "SErver oznámil, že soubor nelze číst.",
                    JOptionPane.ERROR_MESSAGE);
            return;
        }
        displayMessage("\nPřijímám soubor");
        byte buffer[] = new byte[512];
        int nacteno = -1;
        do {
            nacteno = input.read(buffer);
            if (nacteno > 0) {
                displayMessage(new String(buffer, 0, nacteno), fileArea);
            }
        } while (nacteno > 0);
    }

    /**
     * Uzavře proudy a soket.
     */
    private void closeConnection() {
        displayMessage("\nUzavírám spojení.");
        // zrušení možnosti editovat vstupní pole
        try {
            output.close();
            input.close();
            client.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    /**
     * Přidá daný řetězec do textové oblasti. K úpravě dochází v
     * event dispatch vlákně.
     *
     * @param messageToDisplay
     *            zpráva, která se má přidat do textové oblasti.
     * @param area
     *            textová obsat, do které se má zpráva přidat
     */
    private void displayMessage(final String messageToDisplay, final JTextArea area) {
        // final je tam proto, aby se k danému parametru dalo
        // přistoupit z metody vnitřní třídy
        SwingUtilities.invokeLater(new Runnable() {

            public void run() {
                if (null == area) {
                    displayArea.append(messageToDisplay);
                } else {
                    area.append(messageToDisplay);
                }
            }
        });
    }

    private void displayMessage(final String messageToDisplay) {
        displayMessage(messageToDisplay, null);
    }
}
